from unittest.mock import patch

import pytest

from gso.products import Router
from test.workflows import assert_complete, assert_lso_interaction_success, extract_state, run_workflow


@pytest.mark.workflow()
@pytest.mark.parametrize(
    ("remove_configuration", "clean_up_ipam"),
    [
        (True, True),
        (True, False),
        (False, True),
        (False, False),
    ],
)
@patch("gso.services.lso_client._send_request")
@patch("gso.workflows.router.terminate_router.NetboxClient.delete_device")
@patch("gso.workflows.router.terminate_router.infoblox.delete_host_by_ip")
def test_terminate_router_full_success(
    mock_delete_host_by_ip,
    mock_delete_device,
    mock_execute_playbook,
    remove_configuration,
    clean_up_ipam,
    nokia_router_subscription_factory,
    faker,
    data_config_filename,
):
    #  Prepare mock values and expected results
    product_id = nokia_router_subscription_factory()
    router_termination_input_form_data = {
        "tt_number": faker.tt_number(),
        "remove_configuration": remove_configuration,
        "clean_up_ipam": clean_up_ipam,
    }
    lso_interaction_count = 2 if remove_configuration else 0

    #  Run workflow
    initial_router_data = [{"subscription_id": product_id}, router_termination_input_form_data]
    result, process_stat, step_log = run_workflow("terminate_router", initial_router_data)

    for _ in range(lso_interaction_count):
        result, step_log = assert_lso_interaction_success(result, process_stat, step_log)

    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = Router.from_subscription(subscription_id)

    assert subscription.status == "terminated"
    assert mock_delete_device.call_count == 1
    assert mock_delete_host_by_ip.call_count == (1 if clean_up_ipam else 0)
    assert mock_execute_playbook.call_count == lso_interaction_count
